@@ -1,7 +1,7 @@
 /*
  * This file is part of uudeview, the simple and friendly multi-part multi-
- * file uudecoder  program  (c)  1994 by Frank Pilhofer. The author may be
- * contacted by his email address,          fp@informatik.uni-frankfurt.de
+ * file uudecoder  program  (c) 1994-2001 by Frank Pilhofer. The author may
+ * be contacted at fp@fpx.de
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -14,7 +14,6 @@
  * GNU General Public License for more details.
  */
 
-
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
@@ -54,7 +53,7 @@
 #include <fptools.h>
 #include <uustring.h>
 
-char * uucheck_id = "$Id: uucheck.c,v 1.10 1996/11/03 12:47:05 fp Exp $";
+char * uucheck_id = "$Id: uucheck.c,v 1.13 2002/02/28 08:30:02 fp Exp $";
 
 /*
  * Arbitrary number. This is the maximum number of part numbers we
@@ -847,7 +846,7 @@ UUPreProcessPart (fileread *data, int *ret)
       }
       lastvalid = 0;
     }
-    else if (lastvalid && data->uudet == lastenc) {
+    else if (lastvalid && data->uudet == lastenc && result->partno == -1) {
       result->subfname = _FP_strdup (uucheck_lastname);
       result->partno   = ++lastpart;
 
@@ -857,6 +856,9 @@ UUPreProcessPart (fileread *data, int *ret)
       if (data->end || (data->partno && data->partno == data->maxpno))
 	lastvalid = 0;
     }
+    else if (data->partno != -1 && result->filename) {
+      result->subfname = _FP_strdup (result->filename);
+    }
     else { 
       /* 
        * it's got no info, it's got no begin, and we don't know anything
@@ -1460,9 +1462,3 @@ UUCheckGlobalList (void)
   return UUGlobalFileList;
 }
 
-
-/*****************************************************************************
- + Frank Pilhofer                             fp@informatik.uni-frankfurt.de +
- +---------------------------------------------------------------------------+
- | Department of Computer Sciences * University of Frankfurt / Main, Germany |
- *****************************************************************************/
