@@ -1,10 +1,10 @@
 #! /bin/sh
 #
 # This is a super-duper do-it-all SFX-install script for uudeview.
-# $Id: wrapper.sh,v 1.13 1997/01/09 14:22:05 fp Exp $
+# $Id: wrapper.sh,v 1.19 2002/04/02 10:06:35 fp Exp $
 #
 VERSION=0.5
-PATCH=13
+PATCH=18
 #
 echo
 echo ----------------------------------------------------------------------
@@ -175,9 +175,9 @@ else
 fi
 
 echo
-echo If you have Tcl version 7.5 or greater, and Tk version 4.1 or greater
-echo installed, UUDeview will add support for a Tcl/Tk extension, plus the
-echo graphical frontend, xdeview.
+echo If you have Tcl/Tk version 8.0 or greater installed, we can try to
+echo build the graphical frontend for UUDeview, xdeview.
+echo
 echo Usually, the configuration script finds out by itself where and if you
 echo have these two packages installed. On the following prompts, you can
 echo either accept the default [y], which means to add support if available,
